@@ -85,19 +85,24 @@ class Samples:
         # label name or csv file are provided
         elif (label_colname is not None) and (csv_file is not None):
 
-            temp = Handler(filename=csv_file).read_from_csv()
+            temp = Handler(filename=csv_file).read_from_csv(return_dicts=True)
+            header = list(temp[0])
 
             # label name doesn't match
-            if any(label_colname in s for s in temp['name']):
-                loc = temp['name'].index(label_colname)
+            if not any(label_colname in s for s in header):
+                loc = header.index(label_colname)
             else:
-                raise ValueError("Label name mismatch.\nAvailable names: " + ', '.join(temp['name']))
+                raise ValueError("Label name mismatch.\nAvailable names: " + ', '.join(header))
+
+            feat_names = header.copy()
+            _ = feat_names.pop(loc)
 
             # read from data dictionary
-            self.x_name = list(elem.strip() for elem in temp['name'][:loc] + temp['name'][(loc + 1):])
-            self.x = np.array(list(feat[:loc] + feat[(loc + 1):] for feat in temp['feature']))
-            self.y = np.array(list(feat[loc] for feat in temp['feature']))
-            self.y_name = temp['name'][loc].strip()
+            self.x_name = feat_names
+            self.x = np.array(list(list(samp_dict[feat_name] for feat_name in feat_names)
+                                   for samp_dict in temp))
+            self.y = np.array(list(samp_dict[label_colname] for samp_dict in temp))
+            self.y_name = label_colname
 
             # if band name dictionary is provided
             if use_band_dict is not None:
@@ -108,8 +113,9 @@ class Samples:
             temp = Handler(filename=csv_file).read_from_csv()
 
             # read from data dictionary
-            self.x_name = list(temp['name'])
-            self.x = np.array(list(feat for feat in temp['feature']))
+            self.x_name = list(temp[0])
+            self.x = np.array(list(list(samp_dict[feat_name] for feat_name in self.x_name)
+                                   for samp_dict in temp))
 
         else:
             ValueError("No data found for label.")
@@ -126,11 +132,9 @@ class Samples:
             if weights_colname is not None:
                 if csv_file is not None:
 
-                    temp = Handler(filename=csv_file).read_from_csv()
-
                     # label name doesn't match
-                    if any(weights_colname in n for n in temp['name']):
-                        loc = temp['name'].index(weights_colname)
+                    if any(weights_colname in n for n in self.x_name):
+                        loc = self.x_name.index(weights_colname)
                     else:
                         raise ValueError("Weight column name mismatch.\nAvailable names: " + ', '.join(temp['name']))
 
